@@ -1,6 +1,6 @@
 r""" models_iosxr.py - Parse, Query, Build, and Modify IOS-style configurations
 
-     Copyright (C) 2021-2022 David Michael Pennington
+     Copyright (C) 2021-2023 David Michael Pennington
      Copyright (C) 2020-2021 David Michael Pennington at Cisco Systems
      Copyright (C) 2019      David Michael Pennington at ThousandEyes
      Copyright (C) 2014-2019 David Michael Pennington at Samsung Data Services
@@ -22,8 +22,6 @@ r""" models_iosxr.py - Parse, Query, Build, and Modify IOS-style configurations
      mike [~at~] pennington [/dot\] net
 """
 
-import traceback
-import sys
 import re
 import os
 
@@ -33,8 +31,11 @@ from ciscoconfparse.ccp_util import (
     _IPV6_REGEX_STR_COMPRESSED1,
     _IPV6_REGEX_STR_COMPRESSED2,
 )
+
+from ciscoconfparse.errors import InvalidCiscoInterface
 from ciscoconfparse.ccp_util import _IPV6_REGEX_STR_COMPRESSED3
 from ciscoconfparse.ccp_util import CiscoRange, IPv4Obj, IPv6Obj
+from ciscoconfparse.ccp_util import CiscoInterface
 from ciscoconfparse.ccp_abc import BaseCfgLine
 
 ### HUGE UGLY WARNING:
@@ -350,7 +351,7 @@ class BaseIOSIntfLine(IOSCfgLine):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.ifindex = None  # Optional, for user use
-        self.default_ipv4_addr_object = IPv4Obj("127.0.0.1/32", strict=False)
+        self.default_ipv4_addr_object = IPv4Obj("0.0.0.1/32", strict=False)
 
     def __repr__(self):
         if not self.is_switchport:
@@ -442,6 +443,21 @@ class BaseIOSIntfLine(IOSCfgLine):
     _INTF_NAME_RE_STR = r"^interface\s+(\S+[0-9\/\.\s]+)\s*"
     _INTF_NAME_REGEX = re.compile(_INTF_NAME_RE_STR)
 
+    @property
+    def interface_object(self):
+        """Return a CiscoInterface() instance for this interface
+
+        Returns
+        -------
+        CiscoInterface
+            The interface name as a CiscoInterface() instance, or '' if the object is not an interface.  The CiscoInterface instance can be transparently cast as a string into a typical Cisco IOS name.
+        """
+        if not self.is_intf:
+            error = "`{self.text}` is not a valid Cisco interface"
+            logger.error(error)
+            raise InvalidCiscoInterface(error)
+        return CiscoInterface("".join(self.text.split()[1:]))
+
     @property
     def name(self):
         r"""Return the interface name as a string, such as 'GigabitEthernet0/1'
@@ -449,7 +465,7 @@ class BaseIOSIntfLine(IOSCfgLine):
         Returns
         -------
         str
-            The interface name as a string, or '' if the object is not an interface.
+            The interface name as a string instance, or '' if the object is not an interface.
 
         Examples
         --------
@@ -485,10 +501,7 @@ class BaseIOSIntfLine(IOSCfgLine):
            'ATM2/0.100'
            >>>
         """
-        if not self.is_intf:
-            return ""
-        name = self.re_match(self._INTF_NAME_REGEX).strip()
-        return name
+        return str(self.interface_object)
 
     @property
     def port(self):
@@ -530,7 +543,7 @@ class BaseIOSIntfLine(IOSCfgLine):
            0
            >>>
         """
-        return self.ordinal_list[-1]
+        return self.interface_object.port
 
     @property
     def port_type(self):
@@ -784,7 +797,7 @@ class BaseIOSIntfLine(IOSCfgLine):
 
     @property
     def ipv4_addr_object(self):
-        r"""Return a ccp_util.IPv4Obj object representing the address on this interface; if there is no address, return IPv4Obj('127.0.0.1/32')"""
+        r"""Return a ccp_util.IPv4Obj object representing the address on this interface; if there is no address, return IPv4Obj('0.0.0.1/32')"""
         try:
             return IPv4Obj("{}/{}".format(self.ipv4_addr, self.ipv4_netmask))
         except DynamicAddressException as e:
@@ -792,9 +805,27 @@ class BaseIOSIntfLine(IOSCfgLine):
         except Exception:
             return self.default_ipv4_addr_object
 
+    @property
+    @logger.catch(reraise=True)
+    def has_no_ipv4(self):
+        r"""Return an ccp_util.IPv4Obj object representing the subnet on this interface; if there is no address, return ccp_util.IPv4Obj('0.0.0.1/32')"""
+        return self.ip_network_object == IPv4Obj("0.0.0.1/32")
+
+    @property
+    @logger.catch(reraise=True)
+    def ip(self):
+        r"""Return an ccp_util.IPv4Obj object representing the subnet on this interface; if there is no address, return ccp_util.IPv4Obj('0.0.0.1/32')"""
+        return self.ipv4_addr_object
+
+    @property
+    @logger.catch(reraise=True)
+    def ipv4(self):
+        r"""Return an ccp_util.IPv4Obj object representing the subnet on this interface; if there is no address, return ccp_util.IPv4Obj('0.0.0.1/32')"""
+        return self.ipv4_addr_object
+
     @property
     def ipv4_network_object(self):
-        r"""Return an ccp_util.IPv4Obj object representing the subnet on this interface; if there is no address, return ccp_util.IPv4Obj('127.0.0.1/32')"""
+        r"""Return an ccp_util.IPv4Obj object representing the subnet on this interface; if there is no address, return ccp_util.IPv4Obj('0.0.0.1/32')"""
         return self.ip_network_object
 
     @property
@@ -1075,7 +1106,7 @@ class BaseIOSIntfLine(IOSCfgLine):
            False
            >>>
         """
-        if not (str(self.ipv4_addr_object.ip) == "127.0.0.1"):
+        if not (str(self.ipv4_addr_object.ip) == "0.0.0.1"):
             try:
                 # Return a boolean for whether the interface is in that
                 #    network and mask
@@ -2043,7 +2074,9 @@ class IOSRouteLine(BaseIOSRouteLine):
             return self.route_info["netmask"]
         elif self._address_family == "ipv6":
             return str(self.network_object.netmask)
-        return retval
+        else:
+            error = f"Address-family unknown: {self._address_family}"
+            raise ValueError(error)
 
     @property
     def masklen(self):
@@ -2232,10 +2265,9 @@ class IOSAaaGroupServerLine(BaseCfgLine):
 
 
 ##
-##-------------  IOS AAA Lines
+##-------------  IOS AAA Login Authentication Lines
 ##
 
-
 class IOSAaaLoginAuthenticationLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -2256,6 +2288,10 @@ class IOSAaaLoginAuthenticationLine(BaseCfgLine):
         return False
 
 
+##
+##-------------  IOS AAA Enable Authentication Lines
+##
+
 class IOSAaaEnableAuthenticationLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -2275,6 +2311,9 @@ class IOSAaaEnableAuthenticationLine(BaseCfgLine):
             return True
         return False
 
+##
+##-------------  IOS AAA Commands Authorization Lines
+##
 
 class IOSAaaCommandsAuthorizationLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
@@ -2296,6 +2335,9 @@ class IOSAaaCommandsAuthorizationLine(BaseCfgLine):
             return True
         return False
 
+##
+##-------------  IOS AAA Commands Accounting Lines
+##
 
 class IOSAaaCommandsAccountingLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
@@ -2318,6 +2360,10 @@ class IOSAaaCommandsAccountingLine(BaseCfgLine):
             return True
         return False
 
+##
+##-------------  IOS AAA Exec Accounting Lines
+##
+
 
 class IOSAaaExecAccountingLine(BaseCfgLine):
     def __init__(self, *args, **kwargs):
